%% script for reproducing the figures of the paper "Couping the Yoccoz-Birkeland population model with price dynamics: chaotic livestock commodities market cycles" by Sylvain Arlot, Stefano Marmi and Duccio Papini (2018--2019)
%%
%% Note: running the whole script (with computations for SP and for SP1) can be long

%%% Initialization
clear all;i_fig=0;

%%% select only a part of the code to be run (1 = do it; 0 = don't do it)
z_calc_SP=1;% make computations for setting SP
z_fig1=1;% draw figure 1 (SP, continuous)
z_fig2and3=1;% draw figures 2--3 (SP, continuous)
z_fig4and5=1;% draw figures 4--5 (SP, 3d)
z_fig7=1;% draw figure 7 (SP, analysis of chaos for P)
z_fig12and13=1;% draw figures 12--13 (SP, fractal dimension)
z_Lyapunov=0;% compute Lyapunov exponents (SP, Nr and P) => calls external script ArMaPa18_Lyapunov.m (where parameters could be set), which requires functions coming from an external matlab package
%
z_calc_SP1=1;% computations for setting SP1
z_fig6=1;% draw figure 6 (SP1, 3d) 

%%% directory where the results / figures files should be stored/loaded
path_to_files='tmp/';

%%% format for the figures
format_ext='.eps';format_str='-depsc2';
%format_ext='.pdf';format_str='-dpdflatex';% for octave only

%%% set font sizes for each figure
fontsize_fig1=10; 
fontsize_fig2and3=15; 
fontsize_fig4and5=15; 
fontsize_fig7=20; 
fontsize_fig6=fontsize_fig4and5; 
fontsize_fig12and13=15; 


%%%%%%
%%%%%% Figures for setting SP (called 'HH' in the code)
%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 1. Computations for setting SP
%% source: main_predefined_calc.m

seed=7;% random seed used for generating the initial conditions
str_expe='HH';%% Note: 'HH' corresponds to the setting called 'SP' in the paper
%fprintf('Experiment %s.\n',str_expe);

% Initialize with a predefined set of parameters
current_dir=path_to_files; 

one_expe_modelBY;
% change window_long & Tmax
options.window_long=10000;
options.Tmax=300000;
options.Tmin=max(0,options.Tmax-max(120,max(options.window_long,options.window_short)));

if z_calc_SP
% Compute a trajectory (requires 1 to 2 hours)
run_modelBY;
end%if z_calc_SP

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 2. Plots for SP: Figures 1, 2, 3, 4, 5, 7, 12 and 13

if (z_fig1 || z_fig2and3 || z_fig4and5 || z_fig7 || z_fig12and13)
% Load the data (if necessary)
tmp_file_name=[path_to_files 'modelBY_' str_expe '_seed' num2str(seed)];
%load([tmp_file_name '_init_param_options.mat'],'init','param','options');
load([tmp_file_name '_output.mat'],'output');
load([tmp_file_name '_output_int.mat'],'output_int');

end%if (z_fig1 || z_fig2and3 || z_fig4and5 || z_fig7 || z_fig12and13)% Load data SP


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 3. Figure 1 (6 plots, continuous time, over the last 50 years)
%% source: fig_cont.m

if z_fig1
window=50;

% Temporal window
tvisu_max=output.t(end);
tvisu_min=max(output.t(1),tvisu_max-window);
%%visu_results(output,param,options,tvisu_min,tvisu_max)

% Prepare the plots
ind_visu=find(output.t>=tvisu_min & output.t <=tvisu_max);
% A0 -> kA0
kA0=max(1,round(options.nb_steps*param.A0));
% A1 -> kA1 (with '-1' because animals die exactly at age A1, not at age A1+time_step)
kA1=max(kA0,round(options.nb_steps*param.A1)-1);
% Omega0 -> kOmega0
kOmega0=round(options.nb_steps*param.Omega0);
% Omega1 -> kOmega1 (with '-1' as for A1)
kOmega1=max(kOmega0,round(options.nb_steps*param.Omega0)-1);
size_init=max(kA1,kOmega1);

% Extraction of data
t_visu=output.t(ind_visu);
Nr_visu=output.Nr(ind_visu);
Br_visu=output.Br(ind_visu);
Bb_visu=output.Bb(ind_visu);
S_visu=output.S(ind_visu);
P_visu=output.P(ind_visu);

% Computation of derived quantities
D_visu=feval(param.D_func,P_visu,param.D_param);
R_visu=feval(param.R_func,P_visu,param.R_param);

% fixed x-axis
t_transl=floor(max(t_visu));
t_visu=t_visu-t_transl;
t_string=['t - ' num2str(t_transl)];
xaxis=[round(min(t_visu)),round(max(t_visu))];

% y-axis for Bb and Br
i_fig=10;figure(i_fig);clf;
plot(t_visu,Br_visu);
yaxisBr=ylim();
close(i_fig);
i_fig=10;figure(i_fig);clf;
plot(t_visu,Bb_visu);
yaxisBb=ylim();
close(i_fig);
yaxisB=[0 max(yaxisBr(2),yaxisBb(2))];

i_fig=0;

% 6 plots at once %%%%%% 
%	1. Nr		2. S & D
%	3. Br		4. P
%	5. Bb		6. R(P)
i_fig=i_fig+1;figure(i_fig);clf;
set(gca,'fontsize',fontsize_fig1)

subplot(3,2,1)
plot(t_visu,Nr_visu);
xlabel(t_string);
ylabel('Nr');
xlim(xaxis);
%
subplot(3,2,3)
plot(t_visu,Br_visu);
xlabel(t_string);
ylabel('Br');
xlim(xaxis);
ylim(yaxisB);
%
subplot(3,2,5)
plot(t_visu,Bb_visu);
xlabel(t_string);
ylabel('Bb');
xlim(xaxis);
ylim(yaxisB);
%
subplot(3,2,2)
plot(t_visu,S_visu,'b-');
hold on;
plot(t_visu,D_visu,'r-');
hold off
xlabel(t_string);
ylabel('S and D');
legend('S(t)','D(t)');
xlim(xaxis);
%
subplot(3,2,4)
plot(t_visu,P_visu);
xlabel(t_string);
ylabel('P');
xlim(xaxis);
%
subplot(3,2,6)
plot(t_visu,R_visu);
xlabel(t_string);
ylabel('R(P(t))');
xlim(xaxis);
%%%%

print([path_to_files 'fig1__modelBY_' str_expe '_seed' num2str(seed) '_6graphs_' num2str(window) format_ext ] , format_str);
close(i_fig);
end%if z_fig1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 4. Figures 2 and 3 (2 x 2 plots, continuous time, over the last 20 years)
%% source: HEb_check_continuous_time.m (adapted to HH)

if z_fig2and3

time_period=20;% length of the time period to be plotted (take something between 10 and 100)

t_precision=1e-5;% precision of the values of t (for testing whether t is an integer or not). Must be strictly smaller than 1/(options.nb_steps).

% Extract information about parameters
Tmax=round(options.Tmax);
tmin=Tmax-time_period;
nb_steps=options.nb_steps;

A1=param.A1;
A0=param.A0;
Omega1=param.Omega1;
Omega0=param.Omega0;

% A0 -> kA0
kA0=max(1,round(nb_steps*A0));
% A1 -> kA1 (with '-1' because animals die exactly at age A1, not at age A1+time_step)
kA1=max(kA0,round(nb_steps*A1)-1);
% Omega0 -> kOmega0
kOmega0=round(nb_steps*Omega0);
% Omega1 -> kOmega1 (with '-1' as for A1)
kOmega1=max(kOmega0,round(nb_steps*Omega1)-1);

% Extract information in continuous time
t_vect_cont=output.t(output.t>=tmin-t_precision);
Nr_vect_cont=output.Nr(output.t>=tmin-t_precision);
S_vect_cont=output.S(output.t>=tmin-t_precision);

Br_vect_cont=output.Br(output.t>=tmin-t_precision-A1);
Bb_vect_cont=output.Bb(output.t>=tmin-t_precision-Omega1);

TotalReprod_vect_cont=zeros(1,numel(Nr_vect_cont));
TotalB_vect_cont=zeros(1,numel(Nr_vect_cont));
newS_vect_cont=zeros(1,numel(Nr_vect_cont));

for j=1:numel(TotalReprod_vect_cont)
	TotalReprod_vect_cont(j)=sum(Br_vect_cont(j+1:kA1+j+1));
	TotalB_vect_cont(j)=sum(Bb_vect_cont(j+1:kOmega1+j+1));
	newS_vect_cont(j)=sum(Bb_vect_cont(j+1:kOmega1+j+1-kOmega0));
end%for j=1:numel(TotalReprod_vect_cont)

% Make some computation for extracting information about non-mature reproducing females

% Extract information at integer times (beginning of summer)

integertimes=(abs(t_vect_cont-round(t_vect_cont)))<t_precision;

Nr_vect_int=Nr_vect_cont(integertimes);
TotalReprod_vect_int=TotalReprod_vect_cont(integertimes);
TotalB_vect_int=TotalB_vect_cont(integertimes);
S_vect_int=S_vect_cont(integertimes);

t_vect_int=(tmin:Tmax);

% Plot the results

% fixed x-axis
t_transl=floor(max(t_vect_cont));
t_vect_cont=t_vect_cont-t_transl;
t_vect_int = t_vect_int-t_transl;
t_string=['t - ' num2str(t_transl)];
xaxis=[round(min(t_vect_cont)),round(max(t_vect_cont))];

% fixed y-axis
yaxis=[0 1.05*max([max(Nr_vect_cont) max(TotalReprod_vect_cont) max(S_vect_cont*((kOmega1-kOmega0+1)/nb_steps)) max(TotalB_vect_cont)])];

%%% Four separate plots for Figures 2 and 3

% Plot of Nr
i_fig=1;
figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig2and3)
plot(t_vect_cont,Nr_vect_cont,'b-',t_vect_int,Nr_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Mature reproducing female population (Nr)');

print([path_to_files 'fig2a__modelBY_' str_expe '_seed' num2str(seed) '_Nr' format_ext ], format_str);
hold off;close(i_fig);

% Plot of Total Reproducing population
i_fig=i_fig+1;
figure(i_fig);clf;hold on;
set(gca,'fontsize',fontsize_fig2and3)
plot(t_vect_cont,TotalReprod_vect_cont,'b-',t_vect_int,TotalReprod_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Total reproducing female population');% (including non-mature)

print([path_to_files 'fig2b__modelBY_' str_expe '_seed' num2str(seed) '_Total_reproducing' format_ext ], format_str);
hold off; close(i_fig);

% Plot of Supply/Population suitable for butchery (differ by the normalizing factor (kOmega1-kOmega0+1)/nb_steps)
i_fig=i_fig+1;
figure(i_fig);clf;hold on;
set(gca,'fontsize',fontsize_fig2and3)

plot(t_vect_cont,S_vect_cont*((kOmega1-kOmega0+1)/nb_steps),'b-',t_vect_int,S_vect_int*((kOmega1-kOmega0+1)/nb_steps),'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Population suitable for butchery (prop to S)');

print([path_to_files 'fig3a__modelBY_' str_expe '_seed' num2str(seed) '_propto_S' format_ext ], format_str);
hold off; close(i_fig);

% Plot of Total Butchery population
i_fig=i_fig+1;
figure(i_fig);clf;hold on;
set(gca,'fontsize',fontsize_fig2and3)
plot(t_vect_cont,TotalB_vect_cont,'b-',t_vect_int,TotalB_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Total butchery population (including youngs)');

print([path_to_files 'fig3b__modelBY_' str_expe '_seed' num2str(seed) '_Total_butchery' format_ext ], format_str);
hold off; close(i_fig);

%%% Single plot for Figures 2 and 3
i_fig=1;
figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig2and3)

% Plot of Nr
subplot(2,2,1);
plot(t_vect_cont,Nr_vect_cont,'b-',t_vect_int,Nr_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Mature reproducing female population (Nr)');

% Plot of Total Reproducing population
subplot(2,2,2);
plot(t_vect_cont,TotalReprod_vect_cont,'b-',t_vect_int,TotalReprod_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Total reproducing female population');% (including non-mature)

% Plot of Supply/Population suitable for butchery (differ by the normalizing factor (kOmega1-kOmega0+1)/nb_steps)
subplot(2,2,3);
plot(t_vect_cont,S_vect_cont*((kOmega1-kOmega0+1)/nb_steps),'b-',t_vect_int,S_vect_int*((kOmega1-kOmega0+1)/nb_steps),'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Population suitable for butchery (prop to S)');

% Plot of Total Butchery population
subplot(2,2,4);
plot(t_vect_cont,TotalB_vect_cont,'b-',t_vect_int,TotalB_vect_int,'ks','markersize',5);
xlabel(t_string);xlim(xaxis);
ylim(yaxis);%a=ylim();ylim([0 a(2)]);
ylabel('Total butchery population (including youngs)');
print([path_to_files 'fig2-3__modelBY_' str_expe '_seed' num2str(seed) format_ext ] , format_str);

hold off; close(i_fig);


end%if z_fig2and3

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 5. Figures 4 and 5: SP, 3d plots for Nr(t) and P(t) at integer times
%% source: fig_3d.m

if z_fig4and5
window=20000;

% Visualization of the results in 3d 
% Choice of a temporal window %%%%%% 
tvisu_min=output_int.t(1);
tvisu_max=output_int.t(end);
tvisu_min=max(tvisu_min,tvisu_max-window);
window_3d=tvisu_max-tvisu_min;

ind_visu=find(output_int.t>=tvisu_min & output_int.t <=tvisu_max);
tvisu_min=min(output_int.t(ind_visu));
tvisu_max=max(output_int.t(ind_visu));

% Extraction of data
Nr_visu=output_int.Nr(ind_visu);
P_visu=output_int.P(ind_visu);

% Plotting data
i_fig=i_fig+1;figure(i_fig);clf;
%set(gca,'fontsize',fontsize_fig4and5)
plot3(Nr_visu(1:end-2),Nr_visu(2:end-1),Nr_visu(3:end),'+','markersize',2);
set(gca,'fontsize',fontsize_fig4and5)
xlabel('Nr(t)');
ylabel('Nr(t+1)');
zlabel('Nr(t+2)');
print([path_to_files 'fig4__modelBY_' str_expe '_seed' num2str(seed) '_3d_Nr_from_' num2str(round(tvisu_min)) '_to_' num2str(round(tvisu_max)) format_ext ] , format_str);
close(i_fig);

i_fig=i_fig+1;figure(i_fig);clf;
%set(gca,'fontsize',fontsize_fig4and5)
plot3(P_visu(1:end-2),P_visu(2:end-1),P_visu(3:end),'+','markersize',2);
set(gca,'fontsize',fontsize_fig4and5)
xlabel('P(t)');
ylabel('P(t+1)');
zlabel('P(t+2)');
print([path_to_files 'fig5__modelBY_' str_expe '_seed' num2str(seed) '_3d_P_from_' num2str(round(tvisu_min)) '_to_' num2str(round(tvisu_max)) format_ext ] , format_str);
close(i_fig);

end%if z_fig4and5


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 6. Figure 7: SP, analysis of chaos for P (5 plots in one figure)
%% source: chaos.m

if z_fig7

% Tunable parameters %%%%%%%%

% consider time series for t>tmin (at least)
tmin=1000;
% values of tau for autocorrelation function (given that 100=1 year): from 0 to tau_max
tau_max=10000;
% maximal admissible absolute value of autocorrelation for tau_star
thresh_autocor=0.1; 
% maximal value of k for computation of the entropy (will be reduced automatically if the time series is not long enough)
kmax_entropy=10;

tau_vect_normalized=(0:tau_max)/(options.nb_steps);

if tmin+200>options.Tmax
	tmin=options.Tmax-200;
end

text_output_file=[path_to_files 'fig7__modelBY_' str_expe '_seed' num2str(seed) '_chaos_P_infos.txt'];
fidtxtchaos=fopen(text_output_file,'a');

str_quant='P';% do chaos analysis for the price
str_quant_joli='price';

fprintf('\nStudy of chaos in the %s for model SP\n',str_quant_joli);
fprintf(fidtxtchaos,'\n\nStudy of chaos in the %s for model SP (%s)\n',str_quant_joli,datestr(now));

	%%% prepare the time series (enlarge tmin if possible)
	tmin=max(tmin,options.Tmax-options.window_3d);
	Y=output.(str_quant)(output.t>tmin);

	%%% compute and plot the autocorrelation function
	r_vect=autocor(Y,tau_max);

% compute tau_star, the first value of tau for which autocorrelation<0

tau_star=min(find(r_vect<0));
if numel(tau_star)==0
	tau_star=options.nb_steps+1;
	z_tau_star_undefined=1;
	str_print='\n The autocorrelation is always >0. We choose by convention tau_star = %1.3g   =>  autocorrelation is %1.3g \n';
else
	z_tau_star_undefined=0;
	str_print='\ntau_star = %1.3g  =>  autocorrelation is %1.3g \n';
	if tau_star>floor(numel(Y)/2)
		tau_star=floor(numel(Y)/2);
		str_print_bis=[str_print 'But tau_star is too small compared to the length of the time series, so we reduce its value. Be careful with the results!\n'];
	end%if tau_star>floor(numel(Y)/2)
end%if numel(tau_star)==0

fprintf(str_print,(tau_star-1)/(options.nb_steps),r_vect(tau_star));
fprintf(fidtxtchaos,str_print,(tau_star-1)/(options.nb_steps),r_vect(tau_star));

res_chaos.(str_quant).tau_star=tau_star;
res_chaos.(str_quant).tau_star_joli=(tau_star-1)/(options.nb_steps);

if abs(r_vect(tau_star))<thresh_autocor
	fprintf('This value is small enough, we continue.\n\n');
	fprintf(fidtxtchaos,'This value is small enough, we continue.\n\n');
else
	fprintf('This value is too large, but we still continue. ATTENTION! \n\n');
	fprintf(fidtxtchaos,'This value is too large, but we still continue. ATTENTION! \n\n');
end%if abs(r_vect(tau_star))<thresh_autocor

%%% Set of values of tau to be considered for entropy computation

if z_tau_star_undefined
	tries_tau=[options.nb_steps+1];
else
	tries_tau=[tau_star options.nb_steps+1];
end%if z_tau_star_undefined

%%% Plot autocorrelation function

i_fig=i_fig+1;figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig7)
%%subplot(numel(tries_tau),3,1);hold on;
%
plot(tau_vect_normalized,r_vect,'b-');
plot(tau_vect_normalized,zeros(size(tau_vect_normalized)),'r-');
xlabel('tau (years)');
ylabel(['autocorrelation function of ' str_quant_joli]);
print([path_to_files 'fig7a__modelBY_' str_expe '_seed' num2str(seed) '_chaos_' str_quant '_Auto_corr' format_ext ], format_str);
hold off;close(i_fig); 

%%%%%% Entropy computation: try tau_star and 100 (ie, 1 year)

for i_tau=1:numel(tries_tau)
	tau=tries_tau(i_tau);
	res_chaos.(str_quant).tau_vect(i_tau)=tau;
	res_chaos.(str_quant).tau_joli_vect(i_tau)=(tau-1)/(options.nb_steps);

%%% compute the returns and plot them over a few years

return_vect=log(Y(tau+1:tau:end))-log(Y(1:tau:end-tau));
eps_vect=(sign(return_vect)+1)/2;

i_fig=i_fig+1;figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig7)
%%subplot(numel(tries_tau),3,3*i_tau-1);hold on;
%
plot(1:numel(return_vect),return_vect,'+','markersize',2);
plot(1:numel(return_vect),zeros(1,numel(return_vect)),'r-');
xlabel(['time (multiples of tau = ' num2str((tau-1)/(options.nb_steps)) ')']);
ylabel(['returns of ' str_quant_joli]);
print([path_to_files 'fig7b' num2str(i_tau) '__modelBY_' str_expe '_seed' num2str(seed) '_chaos_' str_quant '_Returns' format_ext ], format_str);
hold off;close(i_fig); 


%%% compute and plot the entropy(k) of eps_vect
kmax=min(kmax_entropy,floor(log(numel(eps_vect))/log(2)));
entropy_vect=entropy_binary(eps_vect,kmax);
k_vect=1:kmax;


if kmax<2
	str_print='There is not enough data for trying to estimate an entropy, probably because of tau_star.\n We stop here for chaos.m.\n';
	fprintf(str_print);
	fprintf(fidtxtchaos,[str_print '\n']);
	error(str_print);
end%if kmax<2

%%% least-squares fit
[ls_entropy ls_intercept ls_corr_coeff]=linreg(k_vect,entropy_vect);
str_print='Entropy estimation for the %s (with tau = %1.3g): %1.3g (correlation coefficient = %1.3g)\n';
fprintf(str_print,str_quant_joli,(tau-1)/(options.nb_steps),ls_entropy,ls_corr_coeff);
fprintf(fidtxtchaos,str_print,str_quant_joli,(tau-1)/(options.nb_steps),ls_entropy,ls_corr_coeff);

res_chaos.(str_quant).ls_entropy_vect(i_tau)=ls_entropy;
res_chaos.(str_quant).ls_corr_coeff_vect(i_tau)=ls_corr_coeff;

%%% robust least squares fit (if run with matlab)
%brob = robustfit(k_vect,entropy_vect');
%fprintf('Robust least-squares estimation of entropy: %1.3g\n',brob(2));

i_fig=i_fig+1;figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig7)
%%subplot(numel(tries_tau),3,3*i_tau);hold on;
plot(1:kmax,entropy_vect,'b-','LineWidth',2);%,'markersize',2);
plot(k_vect,ls_intercept+ls_entropy*k_vect,'r','LineWidth',2);
%plot(k_vect,brob(1)+brob(2)*k_vect,'g','LineWidth',2);
legend('Data',['OLS: slope ' num2str(ls_entropy)],'location','northwest');%,'Robust Regression')
xlabel('k');
ylabel('entropy (k)');
title(['Entropy with tau = ' num2str((tau-1)/(options.nb_steps))]);
print([path_to_files 'fig7c' num2str(i_tau) '__modelBY_' str_expe '_seed' num2str(seed) '_chaos_' str_quant '_Entropy' format_ext ], format_str);
hold off;close(i_fig); 

fprintf(fidtxtchaos,'\n\n');
end%for i_tau=1:2

%%print([path_to_files 'fig7' num2str(i_tau) '__modelBY_' str_expe '_seed' num2str(seed) '_chaos_' str_quant format_ext ] , format_str);
% Close the txt file
fclose(fidtxtchaos);

end%z_fig7

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 7. Figures 12 and 13: SP, compute fractal dimension of the attractor in 3d
%% source: dim_f.m 

if z_fig12and13
    
%%% 7a. Figure 12: for Nr
    %extract data to be analyzed (3d point cloud)
    popu_3t0=output_int.Nr(end-200002:end-2);
    popu_3t1=output_int.Nr(end-200001:end-1);
    popu_3t2=output_int.Nr(end-200000:end);

lograyons=(-2:0.01:-0.5);
rayons=10.^lograyons;

% nb of boxes required to cover the attractor for each box size ('rayon')
tailles=zeros(size(rayons));
for i=1:size(rayons,2)
    rayon=rayons(i);
	tmp_r=zeros(size(popu_3t0,2),3);
	tmp_r(:,1)=floor(popu_3t0'/rayon);
	tmp_r(:,2)=floor(popu_3t1'/rayon);
	tmp_r(:,3)=floor(popu_3t2'/rayon);
	tmp_r2=unique(tmp_r,'rows');
    tailles(i)=size(tmp_r2,1);
end%for i=1:size(rayons,2)

%linear regression (restricted to a smart set of the possible box sizes)
indices=(1:size(rayons,2));
i0=min(indices(tailles<size(popu_3t0,2)/10));
r0=rayons(i0);
i1=indices(abs(rayons-r0^0.5)==min(abs(rayons-r0^0.5)));
r1=rayons(i1);
lograyons=log(rayons(i0:i1))/log(10);
logtailles=log(tailles(i0:i1))/log(10);
t=lograyons';
y=logtailles';
X=[ones(size(t)) t];
a=X\y;
frac_dim_estimated=-a(2);

i_fig=i_fig+1;figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig12and13)
plot(lograyons,logtailles,'bx','markersize',2);
plot([min(lograyons) max(lograyons)],a(1)+a(2)*[min(lograyons) max(lograyons)],'r-','LineWidth',2); 
legend('original data',['linear regression (slope ' num2str(a(2)) ')'],'location','southwest');
xlabel('log10 (r)');
ylabel('log10 (N(r))');
title('Evaluation of the fractal dimension for Nr');
print([path_to_files 'fig12__modelBY_' str_expe '_seed' num2str(seed) '_frac_dim_Nr' format_ext ], format_str);
hold off;close(i_fig); 


%%% 7b. Figure 13: for P
    %extract data to be analyzed (3d point cloud)
    popu_3t0=output_int.P(end-200002:end-2);
    popu_3t1=output_int.P(end-200001:end-1);
    popu_3t2=output_int.P(end-200000:end);

lograyons=(-2:0.01:-0.5);
rayons=10.^lograyons;

% nb of boxes required to cover the attractor for each box size ('rayon')
tailles=zeros(size(rayons));
for i=1:size(rayons,2)
    rayon=rayons(i);
	tmp_r=zeros(size(popu_3t0,2),3);
	tmp_r(:,1)=floor(popu_3t0'/rayon);
	tmp_r(:,2)=floor(popu_3t1'/rayon);
	tmp_r(:,3)=floor(popu_3t2'/rayon);
	tmp_r2=unique(tmp_r,'rows');
    tailles(i)=size(tmp_r2,1);
end%for i=1:size(rayons,2)

%linear regression (restricted to a smart set of the possible box sizes)
indices=(1:size(rayons,2));
i0=min(indices(tailles<size(popu_3t0,2)/10));
r0=rayons(i0);
i1=indices(abs(rayons-r0^0.5)==min(abs(rayons-r0^0.5)));
r1=rayons(i1);
lograyons=log(rayons(i0:i1))/log(10);
logtailles=log(tailles(i0:i1))/log(10);
t=lograyons';
y=logtailles';
X=[ones(size(t)) t];
a=X\y;
frac_dim_estimated=-a(2);

i_fig=i_fig+1;figure(i_fig);clf;hold on; 
set(gca,'fontsize',fontsize_fig12and13)
plot(lograyons,logtailles,'bx','markersize',2);
plot([min(lograyons) max(lograyons)],a(1)+a(2)*[min(lograyons) max(lograyons)],'r-','LineWidth',2); 
legend('original data',['linear regression (slope ' num2str(a(2)) ')'],'location','southwest');
xlabel('log10 (r)');
ylabel('log10 (N(r))');
title('Evaluation of the fractal dimension for P');
print([path_to_files 'fig13__modelBY_' str_expe '_seed' num2str(seed) '_frac_dim_P' format_ext ], format_str);
hold off;close(i_fig); 

end%if z_fig12and13 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 8. Compute Lyapunov exponents

if z_Lyapunov
    ArMaPa18_Lyapunov; 
end%if z_Lyapunov

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%
%%%%%%
%%%%%%


%%%%%%
%%% 11. Computations for setting SP1
%% source: main_predefined_calc.m

seed=7;% random seed used for generating the initial conditions
%%str_expe='JA';%% Note: 'JA' corresponds to the setting called 'SP1' in the paper
%fprintf('Experiment %s.\n',str_expe);

if z_calc_SP1
str_expe='HH'; 
% Load the data from SP
tmp_file_name=[path_to_files 'modelBY_HH_seed' num2str(seed)];
load([tmp_file_name '_init_param_options.mat'],'init','param','options');
load([tmp_file_name '_output.mat'],'output');
load([tmp_file_name '_output_int.mat'],'output_int');

% Compute the average of R(P(t)) over 290 000 \leq t \leq 300 000 in the simulation experiments of SP
window=100000;
tvisu_max=output.t(end);
tvisu_min=max(output.t(1),tvisu_max-window);
ind_visu=find(output.t>=tvisu_min & output.t <=tvisu_max);
P_visu=output.P(ind_visu);
R_visu=feval(param.R_func,P_visu,param.R_param);
R_cst_SP1=mean(R_visu); 

% Initialize with the predefined set of parameters for SP
str_expe='HH'; 
current_dir=path_to_files; 
one_expe_modelBY;
% Make R constant => get SP1
param.R_func=@R_cst;
R_param.R0=R_cst_SP1;
param.R_param=R_param;
str_expe='JA'; 
options.root_file_name=['modelBY_' str_expe];
tmp_file_name=[current_dir options.root_file_name '_seed' num2str(seed)];

% change window_long & Tmax
options.window_long=5000;
options.Tmax=100000;
options.Tmin=max(0,options.Tmax-max(120,max(options.window_long,options.window_short)));

% Compute a trajectory (requires 1 to 2 hours)
run_modelBY;
end%if z_calc_SP1

%%%%%%
%%% 12. Plots for SP1: Figure 6

if z_fig6
str_expe='JA';
% Load the data (if necessary)
tmp_file_name=[path_to_files 'modelBY_' str_expe '_seed' num2str(seed)];
load([tmp_file_name '_init_param_options.mat'],'init','param','options');
load([tmp_file_name '_output.mat'],'output');
load([tmp_file_name '_output_int.mat'],'output_int');

end%if z_fig6 (load data SP1)


%%%%%%
%%% 13. Figure 6: SP1, 3d plot for Nr(t) at integer times
%% source: fig_3d.m

if z_fig6
window=5000;

% Visualization of the results in 3d 
% Choice of a temporal window %%%%%% 
tvisu_min=output_int.t(1);
tvisu_max=output_int.t(end);
tvisu_min=max(tvisu_min,tvisu_max-window);
window_3d=tvisu_max-tvisu_min;

ind_visu=find(output_int.t>=tvisu_min & output_int.t <=tvisu_max);
tvisu_min=min(output_int.t(ind_visu));
tvisu_max=max(output_int.t(ind_visu));

% Extraction of data
Nr_visu=output_int.Nr(ind_visu);
%P_visu=output_int.P(ind_visu);

% Plotting data
i_fig=i_fig+1;figure(i_fig);clf;
%set(gca,'fontsize',fontsize_fig6)
plot3(Nr_visu(1:end-2),Nr_visu(2:end-1),Nr_visu(3:end),'+','markersize',2);
set(gca,'fontsize',fontsize_fig6)
xlabel('Nr(t)');
ylabel('Nr(t+1)');
zlabel('Nr(t+2)');
print([path_to_files 'fig6__modelBY_' str_expe '_seed' num2str(seed) '_3d_Nr_from_' num2str(round(tvisu_min)) '_to_' num2str(round(tvisu_max)) format_ext ] , format_str);
close(i_fig);

end%if z_fig6





%%%%%%
%%%%%%
%%%%%%


