%%function [output,output_int,param]=modelBY(init,param,options)
% This function iterates the dynamics of model BY over some time period (defined in 'options'), with initial conditions 'init' and parameters given by 'param'
% INPUTS: 
%	init: structure containing initial conditions
%		init.t: vector of time positions (in years, important for the seasonality factor)
%				t means 'the interval (t-time_step,t]'
%		init.S: vector of the supply [normalized]
%		init.Nr: vector of the (reproducing female) population [normalized]
%		init.Br: vector of the newborns going into the reproducing line [normalized]
%		init.Bb: vector of the newborns going into the butchery line [normalized]
%		init.P: vector of the prices [normalized]
%				=> each value intends to be an average value over each interval [t,t+time_step)
%	!!!IMPORTANT!!! these 4 vectors must have the same length, equal to (options.nb_steps)*max(param.A1,param.Omega1)
%
%	param: structure containing parameters of the model
%		param.A0: maturation age (minimal age taken into account in Nf, among the 'reproducing line')
%		param.A1: maximal age (maximal age taken into account in Nf, among the 'reproducing line')
%		param.Omega0: minimal age taken into account in the supply S (among the 'butchery line')
%		param.Omega1: maximal age taken into account in the supply S (among the 'butchery line')
%		param.m0: average number of female babies per year for a female of age >A0
%		param.sexratio: sex ratio (number of female babies / total number of babies) - default:1/2 - in general \in [0,1]
%       param.gamma: exponent of the nonlinear function that gives reproductivity wrt N
%       param.Nthresh: threshold at which the reproductivity changes
%
%		param.rho_func: function used for computing rho(t). Its syntax must be z=rho_func(t,rho_param). 
%		param.rho_param: parameters to be given to the function rho_func
%
%		param.D_func: function used for computing D(P). Its syntax must be z=D_func(P,D_param). 
%		param.D_param: parameters to be given to the function D_func
%
%		param.R_func: function used for computing R(P). Its syntax must be z=R_func(P,R_param). 
%		param.R_param: parameters to be given to the function R_func
%
%		param.F_func: function used for computing F(D,S). Its syntax must be z=F_func(D,S). 
%		param.lambda: multiplicative factor in front of F(D,S) in the market price dynamics
%
%	options: structure containing parameters of the computation
%		options.Tmax: maximal time of the simulation
%		options.Tmin: minimal storage time in the (continous) output. If necessary, options.Tmin is lowered so that Tmax-Tmin>size_init (i.e., keep the possibility to continue the simulation from the end)
%		options.nb_steps: number of discretization steps per year (positive integer)
%		
% OUTPUT: 
%	output: structure containing the trajectory of the model in continuous time, for t=options.Tmin to options.Tmax
%		output.t: vector of time positions (in years)
%		output.S: vector of the supply [normalized]
%		output.Nr: vector of the (reproducing female) population [normalized]
%		output.Br: vector of the newborns going into the reproducing line [normalized]
%		output.Bb: vector of the newborns going into the butchery line [normalized]
%		output.P: vector of the prices [normalized]
%
%	output_int: structure containing the trajectory of the model once per year (when mod(t,1)=0) for t=0 (max(init.t)) to options.Tmax
%		output_int.t: vector of time positions (in years)
%		output_int.S: vector of the supply [normalized]
%		output_int.Nr: vector of the (reproducing female) population [normalized]
%		output_int.P: vector of the prices [normalized]
%
%	param: structure containing the parameters of the model, same as the input one, with possible modifications (roundings, etc.)

%%%%%%%%%% 1. Check some conditions on the inputs

%%%%%%% 1.1 Default values for the output if some test fails

output.t=NaN;
output.S=NaN;
output.Nr=NaN;
output.Br=NaN;
output.Bb=NaN;
output.P=NaN;

output_int.t=NaN;
output_int.S=NaN;
output_int.Nr=NaN;
output_int.P=NaN;

%%%%%%% 1.2 Discretize all time-related parameters
%%%%%%% t... -> k...

time_step=1/(options.nb_steps);

%%% Tmin -> kmin = 1 (not useful)
%%kmin=1;

%%% Tmax -> kmax
%Tzero=min(init.t);
kmax=1+round((options.Tmax-min(init.t))*options.nb_steps);

%%% parameters of the model
%%% A0 -> kA0
kA0=max(1,round(options.nb_steps*param.A0));
%%% A1 -> kA1 (with '-1' because animals die exactly at age A1, not at age A1+time_step)
kA1=max(kA0,round(options.nb_steps*param.A1)-1);
%%% Omega0 -> kOmega0
kOmega0=round(options.nb_steps*param.Omega0);
%%% Omega1 -> kOmega1 (with '-1' as for A1)
kOmega1=max(kOmega0,round(options.nb_steps*param.Omega1)-1);

%%% Change param in order to show in the output that some parameters have been rounded
param.A0=kA0/(options.nb_steps);
param.A1=(kA1+1)/(options.nb_steps);
param.Omega0=kOmega0/(options.nb_steps);
param.Omega1=(kOmega1+1)/(options.nb_steps);


%%%%%%% 1.3 all elements of init must have the same size  = max(kA1,kOmega1)

size_init=max(kA1,kOmega1);

if (numel(init.t) ~= size_init)
fprintf('Error in modelB.m: size of init.t incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.t));return;
end

if (numel(init.S) ~= size_init)
fprintf('Error in modelB.m: size of init.S incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.S));return;
end

if (numel(init.Nr) ~= size_init)
fprintf('Error in modelB.m: size of init.Nr incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.Nr));return;
end

if (numel(init.Br) ~= size_init)
fprintf('Error in modelB.m: size of init.Br incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.Br));return;
end

if (numel(init.Bb) ~= size_init)
fprintf('Error in modelB.m: size of init.Bb incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.Bb));return;
end

if (numel(init.P) ~= size_init)
fprintf('Error in modelB.m: size of init.P incorrect. It should be equal to %d. It is equal to %d. \n',size_init,numel(init.P));return;
end


%%%%%%% 1.4 for information, tell whether m0 belongs to the window of 'population dynamics stability'

tmp_fact=1/(2*param.sexratio*(param.A1-param.A0));
if isequal(func2str(param.R_func),'R_cst')
tmp_Rmin=param.R_param.R0;
tmp_Rmax=param.R_param.R0;
elseif isequal(func2str(param.R_func),'R_exp')
tmp_Rmin=param.R_param.R0;
tmp_Rmax=1;
else
tmp_P=[0 Inf 0.1:0.1:10 10.^(1:0.1:10)];
tmp_R=feval(param.R_func,tmp_P,param.R_param);
tmp_Rmin=min(tmp_R);
tmp_Rmax=max(tmp_R);
end
if ((param.m0<tmp_fact/tmp_Rmax) || (param.m0>tmp_fact/tmp_Rmin))
tmp_str='does NOT belong';
else
tmp_str='belongs';
end
fprintf('Window of population dynamics stability:  %1.3g < m0 < %1.3g\nHere, m0 = %1.3g %s to this interval.\n',tmp_fact/tmp_Rmax,tmp_fact/tmp_Rmin,param.m0,tmp_str);



%%%%%%% 1.5 Conditions on options.Tmax and options.Tmin

%%%%%%% 1.5.1 options.Tmax > max(init.t)

if (kmax<=size_init)
fprintf('Error in modelBY.m: k_max = %d (from options.Tmax = %d) is smaller than size_init = %d.\n',kmax,options.Tmax,size_init);
output=init;
return;
end

%%%%%%% 1.5.2 Size of the loop, number of loops -> update Tmax

size_loop=min(kA0,kOmega0);
number_loops=ceil((kmax-size_init)/size_loop);
%%% update Tmax to its true value (no change after this point)
options.Tmax=max(init.t)+number_loops*size_loop/options.nb_steps;


%%%%%%% 1.5.3 Tmin: update Tmin to its true value (with no change after this point)
%%% number of loops to be stored in 'output'
number_loops_output=ceil(options.nb_steps*(options.Tmax-options.Tmin)/size_loop);
number_loops_output=max(number_loops_output,ceil(size_init/size_loop));
options.Tmin=options.Tmax-((number_loops_output*size_loop-0.5)/options.nb_steps);



%%%%%%%%%% 2. Pre-allocations, pre-computations

%%%%%%% 2.2 Pre-allocation of output and output_int
size_output=size_loop*number_loops_output;
output.S=zeros(1,size_output);
output.Nr=zeros(1,size_output);
output.Br=zeros(1,size_output);
output.Bb=zeros(1,size_output);
output.P=zeros(1,size_output);

%%% Pre-allocation of output_int (MODIF Tmin)
%size_output_int=floor(options.Tmax)-ceil(max(init.t)+1/options.nb_steps)+1;
output_int.t=(ceil(max(init.t)+1/options.nb_steps):floor(options.Tmax));
size_output_int=numel(output_int.t);
output_int.S=zeros(1,size_output_int);
output_int.Nr=zeros(1,size_output_int);
output_int.P=zeros(1,size_output_int);

%%% define the 'current index' at which the next data should be put in the output.
current_index=1;
%%% define the 'current index' at which the next data should be put in output_int.
current_index_int=1;

%%%%%%% 2.3 Pre-compute the seasonality factor rho(t) over 1 year

%%%% Precompute the average of rho over ( (i-1)/q , i/q ] for i=1..q
rho_precalc_vect=zeros(1,options.nb_steps);
pas_fin=1e-6;
for i=1:options.nb_steps
	rho_precalc_vect(i)=mean(feval(param.rho_func,((i-1)/options.nb_steps+pas_fin-eps:pas_fin:i/options.nb_steps-eps)',param.rho_param));
end
output.rho_precalc_vect=rho_precalc_vect;

%%%%%%% 2.4 Temporary vectors used in the loop

%%% Initialize data for 1st step of the loop
tmp_loop_t=[init.t ((round(max(init.t)*options.nb_steps))+(1:size_loop))/options.nb_steps];
tmp_loop_S=[init.S zeros(1,size_loop)];
tmp_loop_Nr=[init.Nr zeros(1,size_loop)];
tmp_loop_Br=[init.Br zeros(1,size_loop)];
tmp_loop_Bb=[init.Bb zeros(1,size_loop)];
tmp_loop_P=[init.P zeros(1,size_loop)];

%%% indices of tmp_loop_... that are computed at each round of the loop (from past values)
index_newloop=(size_init+1):(size_init+size_loop);

%%% Initialize the auxiliary vector for the nonlinear reproductivity
aux_loop_Nrgamma=zeros(1,size_loop);

for i_loop=1:number_loops

if options.z_debug
fprintf('i_loop = %d.\n Inner loop',i_loop);
end
%%%%%%% 3.1 Inner loop: compute N_r, S and P
for tmp_current_position=index_newloop

%%% Compute N_r and S at 'tmp_current_position'
tmp_loop_Nr(tmp_current_position)=sum(tmp_loop_Br((tmp_current_position-kA1):(tmp_current_position-kA0)));
tmp_loop_S(tmp_current_position)=sum(tmp_loop_Bb((tmp_current_position-kOmega1):(tmp_current_position-kOmega0)))*(options.nb_steps)/(kOmega1-kOmega0+1);

%%% Compute P at 'tmp_current_position'
%%% P'=lambda*P*F(D(P),S)
%%% 	here, we take (P(t)-P(t-1))/q=lambda*F(D(P(t-1),S(t)))
%%% 	we could do better (numerical integration)
tmp_prev_P=tmp_loop_P(tmp_current_position-1);
tmp_loop_P(tmp_current_position)=max(0,tmp_prev_P+(param.lambda*tmp_prev_P*feval(param.F_func,feval(param.D_func,tmp_prev_P,param.D_param),tmp_loop_S(tmp_current_position)))/(options.nb_steps));

end
%%%%%%% End of Inner loop: compute N_r, S and P
if options.z_debug
fprintf('End of inner loop\n');
end
%%%%%%% 3.2 Compute (at once) B_r and B_b

%%% take the (precomputed) average of rho over the interval ((i-1)/q,i/q]
tmp_rho_loop=rho_precalc_vect(1+mod(round(tmp_loop_t(index_newloop)*options.nb_steps-1),options.nb_steps));

%%% compute the auxiliary vector for the nonlinear reproductivity
for aux_loop=1:size_loop
    if tmp_loop_Nr(size_init+aux_loop)<=param.Nthresh
        aux_loop_Nrgamma(aux_loop)=tmp_loop_Nr(size_init+aux_loop);
    else
        aux_loop_Nrgamma(aux_loop)=tmp_loop_Nr(size_init+aux_loop)^(1-param.gamma);
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if options.z_debug
fprintf('Computing tmp_loop_Br\n');
end

tmp_loop_Br(index_newloop)=(2*param.sexratio*param.m0/(options.nb_steps))*tmp_rho_loop.*aux_loop_Nrgamma.*feval(param.R_func,tmp_loop_P(index_newloop),param.R_param);

if options.z_debug
fprintf('Computing tmp_loop_Bb\n');
end

tmp_loop_Bb(index_newloop)=(param.m0/(options.nb_steps))*tmp_rho_loop.*aux_loop_Nrgamma.*(2-2*param.sexratio*feval(param.R_func,tmp_loop_P(index_newloop),param.R_param));


%%%%%%% 3.3 Save the output of the loop in 'output_int' and 'output' (maybe), and update 'current_index'
if options.z_debug
fprintf('Save in output\n');
end

%%%%%%% save information into output_int
%
%%% Put something into output_int ???
tmp_zzz=tmp_loop_t(index_newloop);
if min(abs(tmp_zzz-round(tmp_zzz)))<1/(100*options.nb_steps)
	tmp_xxx=find(abs(tmp_zzz-round(tmp_zzz))<1/(100*options.nb_steps));
	output_int.S(current_index_int:current_index_int+numel(tmp_xxx)-1)=tmp_loop_S(index_newloop(tmp_xxx));
	output_int.Nr(current_index_int:current_index_int+numel(tmp_xxx)-1)=tmp_loop_Nr(index_newloop(tmp_xxx));
	output_int.P(current_index_int:current_index_int+numel(tmp_xxx)-1)=tmp_loop_P(index_newloop(tmp_xxx));
	current_index_int=current_index_int+numel(tmp_xxx);
end

%%% Put something into output ???
if min(tmp_zzz)>=options.Tmin
	output.t(current_index:(current_index+size_loop-1))=tmp_loop_t(index_newloop);
	output.S(current_index:(current_index+size_loop-1))=tmp_loop_S(index_newloop);
	output.Nr(current_index:(current_index+size_loop-1))=tmp_loop_Nr(index_newloop);
	output.Br(current_index:(current_index+size_loop-1))=tmp_loop_Br(index_newloop);
	output.Bb(current_index:(current_index+size_loop-1))=tmp_loop_Bb(index_newloop);
	output.P(current_index:(current_index+size_loop-1))=tmp_loop_P(index_newloop);
	current_index=current_index+size_loop;
end

if options.z_debug
fprintf('Update current_index\n');
end


%%%%%%% 3.4 Initialize tmp_loop

if i_loop<number_loops
if options.z_debug
fprintf('Initialize tmp_loop\n');
end
%%% computing tmp_loop_t directly from its previous value AND be careful that t stays equal to integers/options.nb_steps
	tmp_loop_t=round(options.nb_steps*(tmp_loop_t+size_loop/options.nb_steps))/options.nb_steps;

	tmp_loop_S=[tmp_loop_S(1+size_loop:end) zeros(1,size_loop)];
	tmp_loop_Nr=[tmp_loop_Nr(1+size_loop:end) zeros(1,size_loop)];
	tmp_loop_Br=[tmp_loop_Br(1+size_loop:end) zeros(1,size_loop)];
	tmp_loop_Bb=[tmp_loop_Bb(1+size_loop:end) zeros(1,size_loop)];
	tmp_loop_P=[tmp_loop_P(1+size_loop:end) zeros(1,size_loop)];
end

if options.z_debug
fprintf('End of step %d of the loop\n',i_loop);
end

end

if options.z_debug
fprintf('End of the computations');
end


